﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Media.Capture;
using Windows.UI.Xaml.Media.Imaging;
using Windows.Storage;
using Windows.Media.MediaProperties;
using Windows.Devices.Enumeration;
using ZXing;
using WinRTXamlToolkit.Imaging;
using Infrastructure.UI;
using System.Diagnostics;
using Windows.Media.Devices;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using WinRTXamlToolkit.IO.Serialization;
using Windows.ApplicationModel.DataTransfer;
using Windows.UI.Core;
using Infrastructure;
using Windows.ApplicationModel.Resources;
using Windows.Storage.Search;

namespace QRCodeScanner
{
    public sealed partial class MainPage : QRCodeScanner.Common.LayoutAwarePage
    {
        DeviceInformation camera;
        MediaDeviceControl focusController;

        MediaElement beep = new MediaElement();
        ResourceLoader loader = new ResourceLoader();

        public MainPage()
        {
            this.InitializeComponent();

            this.EnablePrivacyPolicy(PrivacyPolicy.RunXiaDianQi);

            Window.Current.VisibilityChanged += window_VisibilityChanged;
        }

        protected async override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
            var files = await ApplicationData.Current.LocalFolder.GetFilesAsync();
            files.Where(f => f.Name.EndsWith(".tmp.jpg"));
            foreach (var file in files)
            {
                await file.DeleteAsync();
            }

            var cameras = await DeviceInformation.FindAllAsync(DeviceClass.VideoCapture);

            if (cameras.Count == 0)
            {
                MessageBox.Show(loader.GetString("NoCameraError"));
                return;
            }

            DefaultViewModel["Cameras"] = cameras;

            try
            {
                var id = await XmlSerialization.LoadFromXmlFile<string>("Camera.xml");
                camera = cameras.Where(c => c.Id == id).FirstOrDefault();

                if (camera != null)
                {
                    camerasComboBox.SelectedItem = camera;
                }
            }
            catch
            {
                camerasComboBox.SelectedIndex = 0;
            }

            StorageFolder beepFolder = await Package.Current.InstalledLocation.GetFolderAsync("Assets");
            StorageFile beepFile = await beepFolder.GetFileAsync("Beep.wav");
            var stream = await beepFile.OpenAsync(FileAccessMode.Read);
            beep.AutoPlay = false;
            beep.SetSource(stream, beepFile.ContentType);
        }

        protected override void SaveState(Dictionary<String, Object> pageState)
        {
        }

        private async void StartScan(DeviceInformation camera)
        {
            MediaCapture mc = new MediaCapture();

            await mc.InitializeAsync(
                new MediaCaptureInitializationSettings
                {
                    PhotoCaptureSource = PhotoCaptureSource.Auto,
                    StreamingCaptureMode = StreamingCaptureMode.Video,
                    VideoDeviceId = camera.Id
                });

            focusController = mc.VideoDeviceController.Focus;
            focusController.TrySetAuto(false);
            var maxFocus = focusController.Capabilities.Max;
            focusController.TrySetValue(255);

            previewCaptureElement.Source = mc;
            await mc.StartPreviewAsync();

            while (true)
            {
                try
                {
                    var filename = Guid.NewGuid().ToString() + ".tmp.jpg";

                    var file = await ApplicationData.Current.LocalFolder.CreateFileAsync(filename);
                    await mc.CapturePhotoToStorageFileAsync(ImageEncodingProperties.CreateJpeg(), file);

                    var photo = await new WriteableBitmap(1, 1).LoadAsync(file);

                    await photo.SaveToFile(ApplicationData.Current.LocalFolder, filename);
                    
                    var barcodeReader = new BarcodeReader();
                    
                    var result = barcodeReader.Decode(photo);

                    if (result != null)
                    {
                        resultTextBox.Text = result.Text;
                        beep.Play();
                        await Task.Delay(1000);
                    }

                    await file.DeleteAsync();
                }
                catch
                {
                }
            }
        }

        private void ResetFocus()
        {
            if (focusController != null)
            {
                focusController.TrySetAuto(true);
            }
        }

        protected override void OnGotFocus(RoutedEventArgs e)
        {
            
            base.OnGotFocus(e);
        }

        protected override void OnLostFocus(RoutedEventArgs e)
        {
            
            base.OnLostFocus(e);
        }

        private void window_VisibilityChanged(object sender, VisibilityChangedEventArgs e)
        {
            if (e.Visible)
            {
                if (camera != null)
                {
                    StartScan(camera);
                }
            }
            else
            {
                ResetFocus();
            }
        }

        private void camerasComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            camera = (DeviceInformation)e.AddedItems[0];
            camera.Id.SerializeAsXml("Camera.xml");
            StartScan(camera);
        }

        private void copyButton_Click(object sender, RoutedEventArgs e)
        {
            var dataPackage = new DataPackage();
            dataPackage.RequestedOperation = DataPackageOperation.Copy;
            dataPackage.SetText(resultTextBox.Text);
            
            Clipboard.SetContent(dataPackage);
        }
    }
}
